"use strict";

var _to5Helpers = require("6to5-runtime/helpers");

var _core = require("6to5-runtime/core-js");

module.exports = (function () {
  function Interaction(browser) {
    _to5Helpers.classCallCheck(this, Interaction);

    this._browser = browser;
    // Collects all prompts (alert, confirm, prompt).
    this._prompts = [];
    this._alertFns = [];
    this._confirmFns = [];
    this._confirmCanned = {};
    this._promptFns = [];
    this._promptCanned = {};
  }

  // When alert displayed to user, call this function.
  Interaction.prototype.onalert = function onalert(fn) {
    this._alertFns.push(fn);
  };

  // When prompted with a question, return the response. First argument
  // may be a function.
  Interaction.prototype.onconfirm = function onconfirm(question, response) {
    if (typeof question === "function") this._confirmFns.push(question);else this._confirmCanned[question] = !!response;
  };

  // When prompted with message, return response or null if response is
  // false. First argument may be a function.
  Interaction.prototype.onprompt = function onprompt(message, response) {
    if (typeof message === "function") this._promptFns.push(message);else this._promptCanned[message] = response;
  };

  Interaction.prototype.prompted = function prompted(message) {
    return this._prompts.indexOf(message) >= 0;
  };

  Interaction.prototype.extend = function extend(window) {
    var _this = this;
    // Implements window.alert: show message.
    window.alert = function (message) {
      _this._browser.emit("alert", message);
      _this._prompts.push(message);
      for (var _iterator = _this._alertFns, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);;) {
        var _ref;
        if (_isArray) {
          if (_i >= _iterator.length) break;
          _ref = _iterator[_i++];
        } else {
          _i = _iterator.next();
          if (_i.done) break;
          _ref = _i.value;
        }
        var fn = _ref;
        fn(message);
      }
    };

    // Implements window.confirm: show question and return true/false.
    window.confirm = function (question) {
      _this._browser.emit("confirm", question);
      _this._prompts.push(question);
      var response = _this._confirmCanned[question];
      if (!(response || response === false)) {
        for (var _iterator = _this._confirmFns, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);;) {
          var _ref;
          if (_isArray) {
            if (_i >= _iterator.length) break;
            _ref = _iterator[_i++];
          } else {
            _i = _iterator.next();
            if (_i.done) break;
            _ref = _i.value;
          }
          var fn = _ref;
          response = fn(question);
          if (response || response === false) break;
        }
      }
      return !!response;
    };

    // Implements window.prompt: show message and return value of null.
    window.prompt = function (message, defaultValue) {
      _this._browser.emit("prompt", message);
      _this._prompts.push(message);
      var response = _this._promptCanned[message];
      if (!(response || response === false)) {
        for (var _iterator = _this._promptFns, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);;) {
          var _ref;
          if (_isArray) {
            if (_i >= _iterator.length) break;
            _ref = _iterator[_i++];
          } else {
            _i = _iterator.next();
            if (_i.done) break;
            _ref = _i.value;
          }
          var fn = _ref;
          response = fn(message, defaultValue);
          if (response || response === false) break;
        }
      }
      return response ? response.toString() : response === false ? null : defaultValue || "";
    };
  };

  return Interaction;
})();
//# sourceMappingURL=../zombie/interact.js.map